#######################################################################################################
#####                                                                                            ######
#####                                        Code for:                                           ######
#####                                                                                            ######
#####     Rhemtulla, M., Fried, E. I., Aggen, S. H., Tuerlinckx, F., Kendler, K. S.,             ######
#####     Borsboom, D. (2016). Network analysis of substance abuse and dependence symptoms.      ######
#####     Drug and Alcohol Dependence, 161, 230-237.                                             ######
#####                                                                                            ######
#######################################################################################################



abuse <- read.table("abuseClean.dat", header = TRUE)

library(IsingFit)
library(qgraph)
library(grid)
library(gridBase)
library(gridExtra)
library(ggplot2)
library(lavaan)

load(file = "nodeNames.Rdata")
load(file = "layout.Rdata")
load(file = "layout2.Rdata") #layout for 22-node network
load(file = "legend.Rdata")

shortLegend <- c("A1: Inopportune timing", "A2: Hazardous use", "A3: Legal Consequences", 
                 "A4: Social Problems", "D1: Tolerance", "D2: Withdrawal", 
                 "D3: Use longer than planned", "D4: Unable to stop", 
                 "D5: Monopolizes time", "D6: Interferes with work/life", 
                 "D7: Physical/mental problems")


##### Fit Ising Models for each individual substance
##### We start with estimating 6 individual networks (Fig2), and then average to get 1 global network (Fig1)

### Fit network for each substance
can <- abuse[complete.cases(abuse[,8:18]),8:18] #results in 3838 cases #2224 w skip = 1 excluded
names(can) <- c("A1", "A2", "A3", "A4", "D1", "D2", "D3", "D4", "D5", "D6", "D7")
canNet <- IsingFit(can, plot = FALSE)
canAdj <- canNet$weiadj
canThresh <- canNet$thresholds #log-odds
canProb <- exp(canThresh)/(exp(canThresh)+1) #probabilities

sed <- abuse[complete.cases(abuse[,20:30]),20:30] #results in 800 cases #350 w skip = 1 excluded
names(sed) <- c("A1", "A2", "A3", "A4", "D1", "D2", "D3", "D4", "D5", "D6", "D7")
sedNet <- IsingFit(sed, plot = FALSE)
sedAdj <- sedNet$weiadj
sedThresh <- sedNet$thresholds #log-odds
sedProb <- exp(sedThresh)/(exp(sedThresh)+1) #probabilities

sti <- abuse[complete.cases(abuse[,32:42]),32:42] #results in 1222 cases #671 w skip = 1 excluded
names(sti) <- c("A1", "A2", "A3", "A4", "D1", "D2", "D3", "D4", "D5", "D6", "D7")
stiNet <- IsingFit(sti, plot = FALSE)
stiAdj <- stiNet$weiadj
stiThresh <- stiNet$thresholds #log-odds
stiProb <- exp(stiThresh)/(exp(stiThresh)+1) #probabilities

coc <- abuse[complete.cases(abuse[,44:54]),44:54] #results in 1193 cases #625 w skip = 1 excluded
names(coc) <- c("A1", "A2", "A3", "A4", "D1", "D2", "D3", "D4", "D5", "D6", "D7")
cocNet <- IsingFit(coc, plot = FALSE)
cocAdj <- cocNet$weiadj
cocThresh <- cocNet$thresholds #log-odds
cocProb <- exp(cocThresh)/(exp(cocThresh)+1) #probabilities

opi <- abuse[complete.cases(abuse[,56:66]),56:66] #results in 396 cases #195 w skip = 1 excluded
names(opi) <- c("A1", "A2", "A3", "A4", "D1", "D2", "D3", "D4", "D5", "D6", "D7")
opiNet <- IsingFit(opi, plot = FALSE)
opiAdj <- opiNet$weiadj
opiThresh <- opiNet$thresholds #log-odds
opiProb <- exp(opiThresh)/(exp(opiThresh)+1) #probabilities

hal <- abuse[complete.cases(abuse[,68:78]),68:78] #results in 938 cases #345 w skip = 1 excluded
names(hal) <- c("A1", "A2", "A3", "A4", "D1", "D2", "D3", "D4", "D5", "D6", "D7")
halNet <- IsingFit(hal, plot = FALSE)
halAdj <- halNet$weiadj
halThresh <- halNet$thresholds #log-odds
halProb <- exp(halThresh)/(exp(halThresh)+1) #probabilities

L<- averageLayout(canAdj,sedAdj, stiAdj, cocAdj, opiAdj, halAdj)
max(canAdj,sedAdj, stiAdj, cocAdj, opiAdj, halAdj) # 2.66;  maximum value for plots
substance <- rbind(can, sed, sti, coc, opi, hal)


##### Table 3

thresholds <- round(cbind(canProb, sedProb, stiProb, cocProb, opiProb, halProb),2)
colnames(thresholds) <- c("can", "sed", "sti", "coc", "opi", "hal")

# similarity matrices (correlations between substance adjacency matrices)
canEdge <- vech(canAdj, diagonal = FALSE)
sedEdge <- vech(sedAdj, diagonal = FALSE)
stiEdge <- vech(stiAdj, diagonal = FALSE)
cocEdge <- vech(cocAdj, diagonal = FALSE)
opiEdge <- vech(opiAdj, diagonal = FALSE)
halEdge <- vech(halAdj, diagonal = FALSE)

edgeMat <- cbind(canEdge, sedEdge, stiEdge, cocEdge, opiEdge, halEdge)
colnames(edgeMat) <- c("can", "sed", "sti", "coc", "opi", "hal")
cor(edgeMat)
cor(edgeMat, method="spearman")
cor(thresholds)
apply(edgeMat, 2, var)

### save data
# save(can, file = "can.Rdata")
# save(sed, file = "sed.Rdata")
# save(sti, file = "sti.Rdata")
# save(coc, file = "coc.Rdata")
# save(opi, file = "opi.Rdata")
# save(hal, file = "hal.Rdata")


##### Figure 2: plot Ising Model per substance

canGraph <- qgraph(canAdj, layout = L, maximum = 2.66, #removed 'nodenames = nodenames'
                   vsize = 7, GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

sedGraph <- qgraph(sedAdj, layout = L, maximum = 2.66,
                   vsize = 7, GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

stiGraph <- qgraph(stiAdj, layout = L, maximum = 2.66,
                   vsize = 7, GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

cocGraph <- qgraph(cocAdj, layout = L, maximum = 2.66,
                   vsize = 7, GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

opiGraph <- qgraph(opiAdj, layout = L, maximum = 2.66,
                   vsize = 7, GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

halGraph <- qgraph(halAdj, layout = L, maximum = 2.66,
                   vsize = 7, GLratio = 1,
                   legend = FALSE, normalize = FALSE)


### centrality plot
substance <- rbind(can, sed, sti, coc, opi, hal)
source("centralityPlotII.R") #does not work when package 'reshape' is loaded
centrality1 <- centralityPlotII(list(canGraph, sedGraph, stiGraph, cocGraph, opiGraph, halGraph),
                             standardized = T, relative = F) 

centrality1 <- centrality1 + theme(legend.key.width=unit(1.2, "cm")) #yessssss

centrality1$data$substance <- rep(c("cannabis", "sedatives", "stimulants", "cocaine", "opioids", 
                                    "hallucinogens"), 11) 

pdf("FIGURE2.R.pdf", width=9, height=9)  
  par(mar = c(1, 1, 1, 1), xaxs = "i", yaxs = "i")
  layout(mat = matrix(c(1, 2, 7, 3, 4, 7, 5, 6, 7), 3, 3), 
         widths = c(1, 1, 1), heights = c(1, 1, 1.5))
  plot(canGraph)
  text(paste("Cannabis"), x = -.85, y = 1.1, cex = 1.3)
  plot(sedGraph)
  text(paste("Sedatives"), x = -.85, y = 1.1, cex = 1.3)
  plot(stiGraph)
  text(paste("Stimulants"), x = -.85, y = 1.1, cex = 1.3)
  plot(cocGraph)
  text(paste("Cocaine"), x = -.85, y = 1.1, cex = 1.3)
  plot(opiGraph)
  text(paste("Opioids"), x = -.85, y = 1.1, cex = 1.3)
  plot(halGraph)
  text(paste("Hallucinogens"), x = -.85, y = 1.1, cex = 1.3)
plot.new()
vps <- baseViewports()
pushViewport(vps$figure)        
vp1 <-plotViewport(rep(.01, 4))  
print(centrality1, vp = vp1) 
dev.off()


##### Estimate global Ising Model across substances
# average edges and compute SDs for Figures 1 and 3

### Edge Means and SDs
edgeMean <- matrix(0,11,11)
edgeSD <- matrix(0,11,11)
for(i in 1:11){
  for(j in 1:11) {
    vector  <-  c(canAdj[i,j],sedAdj[i,j],stiAdj[i,j],cocAdj[i,j],opiAdj[i,j],halAdj[i,j])
    edgeMean[i,j] <- mean(vector) 
    edgeSD[i,j] <- sd(vector) 
  } 
} 
#edgeMean: matrix of unweighted mean of edges
#edgeSD: matrix of unweighted SD of edges

### correlations with MLM results
# edgeMean.D<-vech(edgeMean, diagonal=FALSE)
# fixedEffects.D<-vech(fixedEffects2, diagonal=FALSE)
# cor(as.vector(edgeMean.D), as.vector(fixedEffects.D), method="spearman") # 0.92
# cor(as.vector(edgeMean.D), as.vector(fixedEffects.D)) # 0.92
# 
# edgeSD.D<-vech(edgeSD, diagonal=FALSE)
# varMat2.D<-vech(varMat2, diagonal=FALSE)
# cor(as.vector(edgeSD.D), as.vector(varMat2.D)) # 0.48
# cor(as.vector(edgeSD.D), as.vector(varMat2.D), method="spearman") # 0.58


##### Figure 1 
max(edgeMean)  #1.839628
max(edgeMeanW) #1.575005

colnames(edgeMean) <- c("A1", "A2", "A3", "A4", "D1", "D2", "D3", "D4", "D5", "D6", "D7")
colnames(edgeMeanW) <- c("A1", "A2", "A3", "A4", "D1", "D2", "D3", "D4", "D5", "D6", "D7")

fullGraphM <- qgraph(edgeMean, maximum = 1.84, layout = L,
                   vsize = 9, GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

cenPlot <- centralityPlot(fullGraphM, standardized=T, relative=F) # does not work when package 'reshape' is loaded

source(file = "centralityPlotRev.R")
cenPlot <- centralityPlotRev(fullGraphM, standardized=T, relative=F) # does not work when package 'reshape' is loaded 


pdf("FIGURE1.R.pdf", width=7, height=9)  
par(mar = rep(1,4), xaxs = "i", yaxs = "i")
layout(mat = matrix(c(1, 2, 1, 2, 3, 2), 2, 3), widths = c(1, 1))
plot(fullGraphM)
plot.new()
vps <- baseViewports()
pushViewport(vps$figure) ##   I am in the space of the autocorrelation plot
vp1 <-plotViewport(rep(.01, 4)) ## create new vp with margins, you play with this values 
print(cenPlot, vp = vp1) 
plot.new()
legend(x = "center", inset = c(0, 0), bty = "n", #y.intersp = 1.5, 
       legend = shortLegend, cex = 1.2)
dev.off()
dev.off()


##### FIGURE 3
max(edgeSD)  #0.8113942

colnames(edgeSD) <- c("A1", "A2", "A3", "A4", "D1", "D2", "D3", "D4", "D5", "D6", "D7")

fullGraphSD <- qgraph(edgeSD, layout = L, maximum = .81, minimum=.1, edge.color="#666666",
                    vsize = 7, GLratio = 1, legend = FALSE,  normalize = FALSE)

pdf("FIGURE3.R.pdf", width=5, height=5)
plot(fullGraphSD)
dev.off()
dev.off()





##### BOOTSTRAP 

#sample rows from each dataset: #####
#canAdjList <- list()
#canThreshList <- list()
#for (i in 1:500){
#  canSubset <- can[sample(1:nrow(can), size = 500, replace = TRUE), ]
#  canNet.i <- IsingFit(canSubset, plot = FALSE)  
#  canAdjList[[i]] <- canNet.i$weiadj
#  canThreshList[[i]] <- exp(canNet.i$thresholds) / (exp(canNet.i$thresholds)+1)
#  print(i)
#}

#sedAdjList <- list()
#sedThreshList <- list()
#for (i in 1:500){
#  sedSubset <- sed[sample(1:nrow(sed), size = 500, replace = TRUE), ]
#  sedNet.i <- IsingFit(sedSubset, plot = FALSE)  
#  sedAdjList[[i]] <- sedNet.i$weiadj
#  sedThreshList[[i]] <- exp(sedNet.i$thresholds) / (exp(sedNet.i$thresholds)+1)
#  print(i)
#}

#stiAdjList <- list()
#stiThreshList <- list()
#for (i in 1:500){
#  stiSubset <- sti[sample(1:nrow(sti), size = 500, replace = TRUE), ]
#  stiNet.i <- IsingFit(stiSubset, plot = FALSE)  
#  stiAdjList[[i]] <- stiNet.i$weiadj
#  stiThreshList[[i]] <- exp(stiNet.i$thresholds) / (exp(stiNet.i$thresholds)+1)
#  print(i)
#}

#cocAdjList <- list()
#cocThreshList <- list()
#for (i in 1:500){
#  cocSubset <- coc[sample(1:nrow(coc), size = 500, replace = TRUE), ]
#  cocNet.i <- IsingFit(cocSubset, plot = FALSE)  
#  cocAdjList[[i]] <- cocNet.i$weiadj
#  cocThreshList[[i]] <- exp(cocNet.i$thresholds) / (exp(cocNet.i$thresholds)+1)
#  print(i)
#}

#opiAdjList <- list()
#opiThreshList <- list()
#for (i in 1:500){
#  opiSubset <- opi[sample(1:nrow(opi), size = 500, replace = TRUE), ]
#  opiNet.i <- IsingFit(opiSubset, plot = FALSE)  
#  opiAdjList[[i]] <- opiNet.i$weiadj
#  opiThreshList[[i]] <- exp(opiNet.i$thresholds) / (exp(opiNet.i$thresholds)+1)
#  print(i)
#}

#halAdjList <- list()
#halThreshList <- list()
#for (i in 1:500){
#  halSubset <- hal[sample(1:nrow(hal), size = 500, replace = TRUE), ]
#  halNet.i <- IsingFit(halSubset, plot = FALSE)  
#  halAdjList[[i]] <- halNet.i$weiadj
#  halThreshList[[i]] <- exp(halNet.i$thresholds) / (exp(halNet.i$thresholds)+1)
#  print(i)
#}

# #save resampled data adjacency matrices: ####
# save(canAdjList, file = "canAdjList.Rdata")
# save(sedAdjList, file = "sedAdjList.Rdata")
# save(stiAdjList, file = "stiAdjList.Rdata")
# save(cocAdjList, file = "cocAdjList.Rdata")
# save(opiAdjList, file = "opiAdjList.Rdata")
# save(halAdjList, file = "halAdjList.Rdata")
# 
# save(canThreshList, file = "canThreshList.Rdata")
# save(sedThreshList, file = "sedThreshList.Rdata")
# save(stiThreshList, file = "stiThreshList.Rdata")
# save(cocThreshList, file = "cocThreshList.Rdata")
# save(opiThreshList, file = "opiThreshList.Rdata")
# save(halThreshList, file = "halThreshList.Rdata")

load(file = "canAdjList.Rdata")
load(file = "sedAdjList.Rdata")
load(file = "stiAdjList.Rdata")
load(file = "cocAdjList.Rdata")
load(file = "opiAdjList.Rdata")
load(file = "halAdjList.Rdata")
load(file = "canThreshList.Rdata")
load(file = "sedThreshList.Rdata")
load(file = "stiThreshList.Rdata")
load(file = "cocThreshList.Rdata")
load(file = "opiThreshList.Rdata")
load(file = "halThreshList.Rdata")

canAdjMean <- Reduce("+", canAdjList)/length(canAdjList)
canThreshMean <- Reduce("+", canThreshList)/length(canAdjList)
sedAdjMean <- Reduce("+", sedAdjList)/length(sedAdjList)
sedThreshMean <- Reduce("+", sedThreshList)/length(sedAdjList)
stiAdjMean <- Reduce("+", stiAdjList)/length(stiAdjList)
stiThreshMean <- Reduce("+", stiThreshList)/length(stiAdjList)
cocAdjMean <- Reduce("+", cocAdjList)/length(cocAdjList)
cocThreshMean <- Reduce("+", cocThreshList)/length(cocAdjList)
opiAdjMean <- Reduce("+", opiAdjList)/length(opiAdjList)
opiThreshMean <- Reduce("+", opiThreshList)/length(opiAdjList)
halAdjMean <- Reduce("+", halAdjList)/length(halAdjList)
halThreshMean <- Reduce("+", halThreshList)/length(halAdjList)

#plot all networks together, fixing nodes: 
max(canAdjMean); max(sedAdjMean); max(stiAdjMean); max(cocAdjMean); max(opiAdjMean); max(halAdjMean)
#max is 4.09, vs. 2.66 before. 
sedAdjMean[sedAdjMean > 2.66] <- 2.66
opiAdjMean[opiAdjMean > 2.66] <- 2.66

canGraph.BS <- qgraph(canAdjMean, layout = L, maximum = 2.66,
                   vsize = 7, GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

sedGraph.BS <- qgraph(sedAdjMean, layout = L, maximum = 2.66,
                   vsize = 7, GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

stiGraph.BS <- qgraph(stiAdjMean, layout = L, maximum = 2.66,
                   vsize = 7,GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

cocGraph.BS <- qgraph(cocAdjMean, layout = L, maximum = 2.66,
                   vsize = 7,  GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

opiGraph.BS <- qgraph(opiAdjMean, layout = L, maximum = 2.66,
                   vsize = 7, GLratio = 1,
                   legend = FALSE,  normalize = FALSE)

halGraph.BS <- qgraph(halAdjMean, layout = L, maximum = 2.66,
                   vsize = 7, GLratio = 1,
                   legend = FALSE, normalize = FALSE)



### EF: correlate data and BS adjacency matrixes to test for similarity
### remove diagonal values before
can1 <- vech(canAdj, diagonal = FALSE) # remove diagonal from data adjacency matrix
sed1 <- vech(sedAdj, diagonal = FALSE)
sti1 <- vech(stiAdj, diagonal = FALSE)
coc1 <- vech(cocAdj, diagonal = FALSE)
opi1 <- vech(opiAdj, diagonal = FALSE)
hal1 <- vech(halAdj, diagonal = FALSE)
can2 <- vech(canAdjMean, diagonal = FALSE) # remove diagonal from bootstrapped adjacency matrix
sed2 <- vech(sedAdjMean, diagonal = FALSE)
sti2 <- vech(stiAdjMean, diagonal = FALSE)
coc2 <- vech(cocAdjMean, diagonal = FALSE)
opi2 <- vech(opiAdjMean, diagonal = FALSE)
hal2 <- vech(halAdjMean, diagonal = FALSE)

cor(as.vector(can1), as.vector(can2)) # 0.95
cor(as.vector(sed1), as.vector(sed2)) # 0.95
cor(as.vector(sti1), as.vector(sti2)) # 0.93
cor(as.vector(coc1), as.vector(coc2)) # 0.99
cor(as.vector(opi1), as.vector(opi2)) # 0.89
cor(as.vector(hal1), as.vector(hal2)) # 0.93

cor(as.vector(can1), as.vector(can2), method="spearman") # 0.94
cor(as.vector(sed1), as.vector(sed2), method="spearman") # 0.90
cor(as.vector(sti1), as.vector(sti2), method="spearman") # 0.89
cor(as.vector(coc1), as.vector(coc2), method="spearman") # 0.99
cor(as.vector(opi1), as.vector(opi2), method="spearman") # 0.90
cor(as.vector(hal1), as.vector(hal2), method="spearman") # 0.88


### centrality plots bootstrapped networks 
centrality.BS <- centralityPlotII(list(canGraph.BS, sedGraph.BS, stiGraph.BS, cocGraph.BS, opiGraph.BS, halGraph.BS),
                                  standardized = T, relative = FALSE) 
centrality.BS$data$substance <- rep(c("cannabis", "sedatives", "stimulants", "cocaine", "opioids", 
                                      "hallucinogens"), 11) 

### Supplementary Figure: Bootstrapped networks


pdf("Bootstrapped.R.pdf", width=9, height=9)  
par(mar = c(1, 1, 1, 1), xaxs = "i", yaxs = "i")
layout(mat = matrix(c(1, 2, 7, 3, 4, 7, 5, 6, 7), 3, 3), 
       widths = c(1, 1, 1), heights = c(1, 1, 1.5))
plot(canGraph.BS)
text(paste("Cannabis"), x = -.85, y = 1.1, cex = 1.3)
plot(sedGraph.BS)
text(paste("Sedatives"), x = -.85, y = 1.1, cex = 1.3)
plot(stiGraph.BS)
text(paste("Stimulants"), x = -.85, y = 1.1, cex = 1.3)
plot(cocGraph.BS)
text(paste("Cocaine"), x = -.85, y = 1.1, cex = 1.3)
plot(opiGraph.BS)
text(paste("Opioids"), x = -.85, y = 1.1, cex = 1.3)
plot(halGraph.BS)
text(paste("Hallucinogens"), x = -.85, y = 1.1, cex = 1.3)
plot.new()
vps <- baseViewports()
pushViewport(vps$figure) ##   
vp1 <-plotViewport(rep(.01, 4)) ##  
print(centrality.BS, vp = vp1) 
dev.off()
dev.off()


